import asyncio

import uvicorn
from cysystemd import daemon
from cysystemd.daemon import Notification
from fastapi import FastAPI
from fastapi.middleware.httpsredirect import HTTPSRedirectMiddleware

from auth_handlers import auth_router
from common_util import cancel_async_task
from domain_socket_listener import (
    MsgFromGlintServer,
    create_subscriber,
    get_socket_msg,
    listen_for_unix_sock_connections,
)
from general_info_handlers import general_info_router
from glint_session import stop_all_processes_delayed

app = FastAPI()
app.include_router(auth_router)
app.include_router(general_info_router)
app.add_middleware(HTTPSRedirectMiddleware)


async def handle_unix_sock_messages():

    stop_all_processes_task = None
    domain_socket_msg_queue = create_subscriber()
    while True:
        msg = await get_socket_msg(domain_socket_msg_queue)
        if msg == MsgFromGlintServer.USER_CONNECTED.value:
            cancel_async_task(stop_all_processes_task)
            stop_all_processes_task = None

        elif msg == MsgFromGlintServer.USER_DISCONNECTED.value:
            # Start delayed task to stop processes. It's delayed to give user a chance to reconnect
            loop = asyncio.get_event_loop()
            cancel_async_task(stop_all_processes_task)
            stop_all_processes_task = loop.create_task(stop_all_processes_delayed())


@app.on_event("startup")
async def service_tasks_startup():
    print("Starting up ...")  # noqa: T201

    loop = asyncio.get_event_loop()
    asyncio.ensure_future(loop.run_in_executor(None, listen_for_unix_sock_connections))
    loop.create_task(handle_unix_sock_messages())

    # Tell systemd that our service is ready
    daemon.notify(Notification.READY)
    print("Startup complete", flush=True)  # noqa: T201


if __name__ == "__main__":
    # Start server
    uvicorn.run(
        app,
        host="0.0.0.0",
        port=7778,
        log_level="info",
        ssl_keyfile="cert/Private.key",
        ssl_certfile="cert/Certificate.crt",
    )

# Launch
# PYTHONPATH=.
# launch: python3 main.py
